using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Threading;

namespace Demo_4_Suspend_Resume
{
	delegate void CharParamDelegate(char aChar);

	public class MainForm : System.Windows.Forms.Form
	{
		private const string MESSAGE = "This application demonstrates " +
			"Thread.Suspend() and Thread.Resume() methods. ";

		private Thread mThread;

		private System.Windows.Forms.TextBox textBoxMessage;
		private System.Windows.Forms.Button buttonResume;
		private System.Windows.Forms.Button buttonSuspend;
		private System.Windows.Forms.TextBox textBox;

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public MainForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.textBoxMessage = new System.Windows.Forms.TextBox();
			this.buttonResume = new System.Windows.Forms.Button();
			this.buttonSuspend = new System.Windows.Forms.Button();
			this.textBox = new System.Windows.Forms.TextBox();
			this.SuspendLayout();
			// 
			// textBoxMessage
			// 
			this.textBoxMessage.Location = new System.Drawing.Point(16, 16);
			this.textBoxMessage.Multiline = true;
			this.textBoxMessage.Name = "textBoxMessage";
			this.textBoxMessage.ReadOnly = true;
			this.textBoxMessage.Size = new System.Drawing.Size(264, 96);
			this.textBoxMessage.TabIndex = 0;
			this.textBoxMessage.Text = "";
			// 
			// buttonResume
			// 
			this.buttonResume.Location = new System.Drawing.Point(176, 160);
			this.buttonResume.Name = "buttonResume";
			this.buttonResume.TabIndex = 1;
			this.buttonResume.Text = "Resume";
			this.buttonResume.Click += new System.EventHandler(this.buttonResume_Click);
			// 
			// buttonSuspend
			// 
			this.buttonSuspend.Location = new System.Drawing.Point(48, 160);
			this.buttonSuspend.Name = "buttonSuspend";
			this.buttonSuspend.TabIndex = 2;
			this.buttonSuspend.Text = "Suspend";
			this.buttonSuspend.Click += new System.EventHandler(this.buttonSuspend_Click);
			// 
			// textBox
			// 
			this.textBox.Location = new System.Drawing.Point(16, 128);
			this.textBox.Name = "textBox";
			this.textBox.Size = new System.Drawing.Size(264, 20);
			this.textBox.TabIndex = 3;
			this.textBox.Text = "";
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(298, 199);
			this.Controls.Add(this.textBox);
			this.Controls.Add(this.buttonSuspend);
			this.Controls.Add(this.buttonResume);
			this.Controls.Add(this.textBoxMessage);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "MainForm";
			this.Text = "Suspend() and Resume() Demo";
			this.Load += new System.EventHandler(this.MainForm_Load);
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new MainForm());
		}

		private void MainForm_Load(object sender, System.EventArgs e)
		{
			mThread = new Thread(new ThreadStart(this.PrintMessages));
			mThread.IsBackground = true;
			mThread.Start();
			SuspendThread();
		}

		private void SuspendThread()
		{
			mThread.Suspend();
			buttonSuspend.Enabled = false;
			buttonResume.Enabled = true;
		}

		private void ResumeThread()
		{
			mThread.Resume();
			buttonSuspend.Enabled = true;
			buttonResume.Enabled = false;
		}

		private void AppendTextToTextBox(char aChar)
		{
			textBoxMessage.AppendText(aChar.ToString());
		}

		/// <summary>
		/// PrintMessages() runs in a separate thread and slowly
		/// prints messages in the MainForm's text box. 
		/// </summary>
		private void PrintMessages()
		{
			while (true)
			{
				foreach (char letter in MESSAGE.ToCharArray())
				{
					try
					{
						this.Invoke(new CharParamDelegate(AppendTextToTextBox),
							new object[]{letter});
					}
					catch (Exception)
					{
						// Can not call Invoke() bacause the form is closed.
						return;
					}
					Thread.Sleep(50);
				}
			}
		}

		private void buttonSuspend_Click(object sender, System.EventArgs e)
		{
			SuspendThread();
		}

		private void buttonResume_Click(object sender, System.EventArgs e)
		{
			ResumeThread();
		}
	}
}
